
#include <stdio.h>


#include "emospeech.h"
#include "config.h"


/*
@param sig: input waveform
@param op:  options regarding pitch tracking parameters
@param op.min_pitch: minimum permitted F0 value
@param op.max_pitch: maximum permitted F0 value
@param op.pdaFrameShift: analysis frame shift
@param op.pdaFrameLength: analysis frame length
@param op.lpf_cutoff: cut off frequency for low pass filtering
@param op.lpf_order: order of low pass filtering (must be odd)
@param op.decimation
@param op.noise_floor
@param op.min_v2uv_coef_thresh
@param op.v2uv_coef_thresh_ratio
@param op.v2uv_coef_thresh
@param op.anti_doubling_thresh
@param op.peak_tracking

#define MINARG                5
#define BREAK_NUMBER          0.0

#define DEFAULT_DECIMATION    4     // samples 
#define DEFAULT_MIN_PITCH     40.0  // Hz 
#define DEFAULT_MAX_PITCH     400.0 // Hz

#define DEFAULT_SF            20000 // Hz. Sampling Frequency 
#define DEFAULT_SHIFT         5.0   // ms 
#define DEFAULT_LENGTH        10.0   // ms 
#define DEFAULT_TSILENT       120   // max. abs sample amplitude of noise 
#define DEFAULT_TMIN          0.75
#define DEFAULT_TMAX_RATIO    0.85
#define DEFAULT_THIGH         0.88
#define DEFAULT_TDH           0.77

#define UNVOICED              0     // segment classifications 
#define VOICED                1
#define SILENT                2

#define HOLD                  1
#define HELD                  1
#define SEND                  2
#define SENT                  2

*/
	void EmoData::process()
	{
		/// normalization of the wave
		wave.rescale(config.normalizeGain, 0);

		/// create pitch data  structure
		real processed_time = wave.num_samples()/(real)wave.sample_rate() - config.pdaFrameLength + config.pdaFrameShift;
		int f0size = (int)(processed_time/config.pdaFrameShift);
		if (f0.length() != f0size) 
		{ 
			f0.resize(f0size, 1); 
			f0.set_equal_space(true);
			f0.fill_time(config.pdaFrameShift, time);
		}
	
		/// analyze pitch
		// get pitch
		EST_Features op_features;
		op_features.set("pda_frame_shift", config.pdaFrameShift);
		op_features.set("pda_frame_length", config.pdaFrameLength);
		op_features.set("noise_floor", config.noiseFloor);
		pda(wave, f0, op_features);
		// smoothing+filtering
//		EST_Option op_option;
//		smooth_portion(f0, op_option);
		// average
		pitch_avg = 0;
		int num_talked = 0;
		for(int i = 0; i < f0.length(); i++)
			if (f0(i) > 0.001) { num_talked++; pitch_avg += f0(i); }
		if (num_talked > 0) pitch_avg /= num_talked;
		pct_voiced = num_talked / (real)f0.length();
		// var
		pitch_sqr = 0;
		for(int i = 0; i < f0.length(); i++)
			if (f0(i) > 0.001) pitch_sqr += sqr(f0(i));
		if (num_talked > 0) pitch_sqr /= num_talked;
		pitch_var = pitch_sqr - sqr(pitch_avg);
		
		
		/// energy
		// get energy
		if (engy.length() != f0.length()) 
		{ 
			engy.resize(f0.length(), 1); 
			engy.set_equal_space(true);
			engy.fill_time(config.pdaFrameShift, time);
		}
		//energy(wave, engy, 1);
		int samples_length = (int)(wave.sample_rate() * config.pdaFrameLength);
		for(int i = 0; i < engy.length(); i++) 
		{
			double sum = 0;
			int val;
			int start = (int)(i * (wave.sample_rate() * config.pdaFrameShift));
			for(int j = 0; j < samples_length; j++)
			{
				val = wave(start+j); 
				sum += sqr(val);
			}
			real engy_ = sqrt(sum/samples_length);
			engy(i) = engy_;
		}

		// average
		energy_avg = 0;
		for(int i = 0; i < engy.length(); i++)
			if (f0(i) > 0.001) energy_avg += engy(i);
		if (num_talked > 0) energy_avg /= num_talked;
		// var
		energy_sqr = 0;
		for(int i = 0; i < engy.length(); i++)
			if (f0(i) > 0.001) energy_sqr += sqr(engy(i));
		if (num_talked > 0) energy_sqr /= num_talked;
		energy_var = energy_sqr - sqr(energy_avg);
	}



int EmoSpeech::computeGlobalFeatures()
{
	/// compute global features
	real pct_voiced=0;
	real pitch_avg=0, pitch_sqr=0, pitch_stdev;
	real energy_avg=0, energy_sqr=0, energy_stdev, energy_normdev;
	
	int i = 0;
	while (true)
	{
		EmoData &ed = as->getWave(i); if (ed.wave.length() == 0) break; 
		
		pct_voiced += ed.pct_voiced; 
		pitch_avg += ed.pitch_avg*ed.pct_voiced; 
		pitch_sqr += ed.pitch_sqr*ed.pct_voiced;
		energy_avg += ed.energy_avg*ed.pct_voiced; 
		energy_sqr += ed.energy_sqr*ed.pct_voiced;
		
		i++;
	}
	
	if (i > 0)
	{
		EmoData &ed = as->getWave(i-1);
		printf(clBYELLOW "%02d" clNORMAL " ; " clBWHITE "%03d" clNORMAL ", " clWHITE "%3.1f" clNORMAL " ; " \
			clBWHITE "%05d" clNORMAL ", " clWHITE "%4.1f" clNORMAL " # ",
			(int)(100*ed.pct_voiced), (int)ed.pitch_avg, sqrt(ed.pitch_var), (int)ed.energy_avg, sqrt(ed.energy_var));
	}

	if (i > 0 && pct_voiced > 0)
	{
		pitch_avg /= pct_voiced; pitch_sqr /= pct_voiced;
		energy_avg /= pct_voiced; energy_sqr /= pct_voiced;
		pct_voiced /= i;
	}
	pitch_stdev = sqrt(pitch_sqr - sqr(pitch_avg));
	energy_stdev = sqrt(energy_sqr - sqr(energy_avg));
	energy_normdev = (energy_avg > 0 ? energy_stdev / energy_avg : 0);
	
	features[fPctVoiced] = pct_voiced;
	features[fPitchAvg] = pitch_avg;
	features[fPitchSD] = pitch_stdev;
	features[fEnergyAvg] = energy_avg;
	features[fEnergySD] = energy_stdev;
	features[fEnergyND] = energy_normdev;
	
	return i;
}


EmoSpeech::Emotion EmoSpeech::classify()
{
	// nearest neighbor with malhahanobis distance
	confidence = 0;
	for(int i = 0; i < NEMOTIONS; i++) distances[i] = 0;
	Emotion em;
	if (features[fPctVoiced] < config.minPctVoiced) em = eNothing; else
	{
		for(int i = 0; i < NEMOTIONS; i++)
		{
			for(int j = 0; j < NFEATS; j++)
				distances[i] += sqr((features[j]-config.emfeatCenters[i*NFEATS+j]) / config.emfeatStdevs[i*NFEATS+j]);
			distances[i] = sqrt(distances[i]);
		}
		
		int em1=0;
		for(int i = 1; i < NEMOTIONS; i++)
			if (distances[i] < distances[em1]) em1 = i;
		int em2=(em1==0?1:0);
		for(int i = 0; i < NEMOTIONS; i++)
			if (i != em1 && distances[i] < distances[em2]) em2 = i;
		confidence = (distances[em2]-distances[em1]) / distances[em2];
		if (confidence < config.minConfidence)
			em = eUnknown; else
			em = (Emotion)em1;

/*		
		if (pct_voiced < 0.1) em = eNothing; else
		if (pitch_avg < 150 && pitch_stdev < 35 && energy_normdev < 0.20) em = eSad; else
		if (pitch_avg > 150 && pitch_stdev > 35 && energy_normdev > 0.25) em = eHappy; else
		if (pitch_avg < 150 && pitch_stdev > 30 && energy_normdev > 0.25) em = eAngry; else
			em = eNeutral;
*/
	}
	return em;
}


void EmoSpeech::printChunksFeatures()
{
	as = new AcquireSound<EmoData>(config.nChunks, config.chunkLength, config.acquisFreq, config.acquisDepth);
	as->start();
	
	bool first = true;
	while(true)
	{
		as->waitUpdated();
		int i = 0;
		std::cout << "pct voice / pitch avg/var / enery avg/var: ";
		while (true)
		{
			EmoData &ed = as->getWave(i); if (ed.wave.length() == 0) break; 
		
			printf(clBYELLOW "%02d" clNORMAL "/" clBWHITE "%03d" clNORMAL "/" clWHITE "%3.1f" clNORMAL "/" \
				clBWHITE "%05d" clNORMAL "/" clWHITE "%4.1f" clNORMAL " ",
				(int)(100*ed.pct_voiced), (int)ed.pitch_avg, sqrt(ed.pitch_var), (int)ed.energy_avg, sqrt(ed.energy_var));

			i++;
		}
		std::cout << std::endl;
		
		if (i == config.nChunks && first)
		{
			EST_Option al;
			EST_SingleChannelWave wave(as->getWave(0).wave);
			for(int j = 1; j < config.nChunks; j++) 
			{
				wave += as->getWave(j).wave;
				DEBUG(as->getWave(j).wave.num_samples() << " " << wave.num_samples());
			}
			write_wave(wave, "out.raw", al);
			first = false;
		}
		
	}

}

#define FOREACH_ED(ED, AS, I) \
	int I = 0; while(true) { EmoData &ED = AS.getWave(i); if (ED.wave.length() == 0) break; I++;
	
	
void EmoSpeech::printWindowsFeatures()
{
	as = new AcquireSound<EmoData>(config.nChunks, config.chunkLength, config.acquisFreq, config.acquisDepth);
	as->start();
	
	
	FILE *data_file = fopen("data.csv", "wt");
	fprintf(data_file, "idx\tpct_voiced\tpitch_a\tpitch_d\tengy_a\tengy_d\tengy_nd\n");
	int i_time = 0;
	
	bool first = true;
	while(true)
	{
		as->waitUpdated();
		
		int nchunks = computeGlobalFeatures();
		Emotion em = classify();
				
		/// print them
		if (em != eNothing)
		{
			fprintf(data_file, "%d\t%f\t%f\t%f\t%f\t%f\t%f\n", i_time, 
					features[fPctVoiced], features[fPitchAvg], features[fPitchSD], 
					features[fEnergyAvg], features[fEnergySD], features[fEnergyND]);
			fflush(data_file);
		}
		
		printf(clBYELLOW "%02d" clNORMAL " ; " clBWHITE "%03d" clNORMAL ", " clWHITE "%3.1f" clNORMAL " ; " \
			clBWHITE "%05d" clNORMAL ", " clWHITE "%4.1f" clNORMAL ", %.2f : ",
			(int)(100*features[fPctVoiced]), (int)features[fPitchAvg], features[fPitchSD], 
			(int)features[fEnergyAvg], features[fEnergySD], features[fEnergyND]);
		printEmotion(em);
		std::cout << std::endl<< "distances: ";
		for(int i = 0; i < NEMOTIONS; i++) std::cout << std::setprecision(3) << distances[i] << " ";
		std::cout << "; confidence: " << confidence << std::endl;

		
		/// save first window to check if necessary (na_play out.raw)
		if (nchunks == config.nChunks && first)
		{
			EST_Option al;
			EST_SingleChannelWave wave(as->getWave(0).wave);
			for(int j = 1; j < config.nChunks; j++) 
			{
				wave += as->getWave(j).wave;
				DEBUG(as->getWave(j).wave.num_samples() << " " << wave.num_samples());
			}
			write_wave(wave, "out.raw", al);
			first = false;
		}
		
		i_time++;
	}

}



void EmoSpeech::learnFeatures()
{
/*
	<featureWeights>0;1;1;0;0;1</featureWeights>
	<emfeatCenters>.3,123.59,28.11,13225.31,2991.06,0.22;.3,143.23,27.55,15863.99,3921.38,0.25;.3,163.09,38.86,14093.45,4240.78,0.3;.3,105.15,32.64,9703.56,2059.49,0.21</emfeatCenters>
	<emfeatStdevs>.2,8.25,8.32,1330.73,630.94,0.03;.2,15.06,6.34,1698.79,755.51,0.05;.2,14.04,10.3,2326.8,987.05,0.05;.2,10.61,6.13,716.96,489.58,0.04</emfeatVars>

	<frameLengthShift ignore="1">0.02;0.01</frameLengthShift>
	<EmotionNames ignore="1">sad;happy;angry;neutral</EmotionNames>
	<FeatureNames ignore="1">pct_voiced;pitch_avg;pitch_stdev;energy_avg;energy_stdev;energy_normdev</FeatureNames>
	<ClassificationRate ignore="1">0</ClassificationRate>
	<ConfusionMatrix ignore="1">0,0,0,0;0,0,0,0;0,0,0,0;0,0,0,0</ConfusionMatrix>

	<Data ignore="1" emotion="sad">
*/ /*
	FILE *configfile = fopen(config.learnConfig, "wt");
	fwrite(configfile, "\n\n<EmoSpeechConfig>\n");

	for(int em_ = 0; em_ < NEMOTIONS; em_++)
	{
		std::cout << "Now emotion ";
		printEmotion((Emotion)em_);
		std::cout << std::endl << "please press a key when ready, start talking, \
			and keep talking until told to stop (~ 1 minute)") << std::endl;
		getchar();
		
		fwrite(configfile, "\t<Data ignore=\"1\" emotion=\"sad\">\n");
		
		as = new AcquireSound<EmoData>(config.nChunks, config.chunkLength, config.acquisFreq, config.acquisDepth);
		as->start();
		
		bool first = true;
		int i = 0;
		while(true)
		{
			as->waitUpdated();
			
			computeGlobalFeatures();
			Emotion em = classify();
					
			/// print them
			if (em != eNothing)
			{
				fprintf(configfile, "%f\t%f\t%f\t%f\t%f\t%f\n",  
					features[fPctVoiced], features[fPitchAvg], features[fPitchSD], 
					features[fEnergyAvg], features[fEnergySD], features[fEnergyND]);
			}
			
			printf(clBYELLOW "%02d" clNORMAL " ; " clBWHITE "%03d" clNORMAL ", " clWHITE "%3.1f" clNORMAL " ; " \
				clBWHITE "%05d" clNORMAL ", " clWHITE "%4.1f" clNORMAL,
				(int)(100*features[fPctVoiced]), (int)features[fPitchAvg], features[fPitchSD], 
				(int)features[fEnergyAvg], features[fEnergySD], features[fEnergyND]);
			std::cout << std::endl;
			
			/// save first window to check if necessary (na_play out.raw)
			if (i == nchunks && first)
			{
				EST_Option al;
				EST_SingleChannelWave wave(as->getWave(0).wave);
				for(int j = 1; j < config.nChunks; j++) 
				{
					wave += as->getWave(j).wave;
					DEBUG(as->getWave(j).wave.num_samples() << " " << wave.num_samples());
				}
				write_wave(wave, "out.raw", al);
				first = false;
			}
			
			i_time++;
		}
		std::cout << "You can stop talking, thank you.\n";

	}*/
}


