#ifndef _EMOSPEECH_H_
#define _EMOSPEECH_H_

#include "EST.h"
#include "EST_types.h"
#include "EST_error.h"
#include "EST_SingleChannelWave.h"

#include "config.h"
#include "cyril-macro.h"
#include "AcquireSound.hpp"


struct EmoData
{
	EST_SingleChannelWave wave;
	real time;
	
	EST_Track f0;
	EST_Track engy;
	
	real pct_voiced;
	real pitch_avg, pitch_sqr, pitch_var;
	real energy_avg, energy_sqr, energy_var;

	void process();
};


class EmoSpeech
{
	AcquireSound<EmoData> *as;
	real features[NFEATS];
	real distances[NEMOTIONS];
	real confidence;
	
 public:
	enum Emotion { eNothing=-2, eUnknown=-1, eNeutral=0, eAngry, eHappy, eSad, eNEmotion };
	enum Feature { fPctVoiced=0, fPitchAvg, fPitchSD, fEnergyAvg, fEnergySD, fEnergyND, fNFeature };
	void printEmotion(Emotion em) 
	{ 
		switch(em) 
		{ 
			case eNothing: std::cout << clNORMAL << "_" << clNORMAL; break;
			case eUnknown: std::cout << clNORMAL << "?????" << clNORMAL; break;
			case eNeutral: std::cout << clBWHITE << "Neutral" << clNORMAL; break;
			case eAngry: std::cout << clBRED << "Angry" << clNORMAL; break;
			case eHappy: std::cout << clBGREEN << "Happy" << clNORMAL; break;
			case eSad: std::cout << clBBLUE << "Sad" << clNORMAL; break;
			default: break;
		}
	}
	
	int computeGlobalFeatures();
	Emotion classify();
	
	void printChunksFeatures();
	void printWindowsFeatures();
	void learnFeatures();
};


#endif


