/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2008
*********************************************/

#ifndef CONFIG_PARAMS_H
#define CONFIG_PARAMS_H

//#include "lib.h"
#include "cyril-macro.h"
#include "cyril-std.h"


#ifdef __cplusplus
	extern "C" {
#endif



//#define _USE_XML_


//////////////////////////////////////////////////////////////////
// config xml
//////////////////////////////////////////////////////////////////
// void ____________() {}

/**
A program can have different tools to achieve different tasks.
A tool can have parameters and options :
  - parameters are given in the CLI and are mandatory, thus do not need a name, and their order is important
  - options can be given in the CLI or in an xml config file and have a default value
The syntax to launch the program is :
./program --tool param1 param2 [option1=val1 option2=val2 ...]

Features :
- manages help and usage (tools)
- manages scalars and vectors
- checks data sizes to prevent inconsistencies and memory violations
- possibility to give the same option set to different tools, by giving to them the same configType
- 

	@example
	

*/

extern int argc_;
extern char **argv_;


typedef enum CFG_Type_ { pSep, pBool, pEnum, pInt, pReal, pString, pFile, pConfig, pNType } CFG_Type;
extern const char* CFG_TypeNames[pNType];

typedef struct CFG_Param_
{
	void *data; /// pointer to data
	char *str;
	CFG_Type type; /// base type of the param
	int size; /// size (number of elements of base type)

	char *name; /// name of the param
	char *desc; /// description of the param
	int val; /// when the value was last modified (0 = default, 1 = xml config file, 2 = CLI option
	
	char *range;
	char *default_val;
} CFG_Param;

typedef struct CFG_Tool_
{
	char *name;
	char *desc;
	char *configType;
	
	bool optionsOwner;
	CFG_Param params[2]; // config && help
	CFG_Param *options;
	int optionCount;
	int optionMaxCount;
} CFG_Tool;

extern int CFG_toolCount;
extern int CFG_toolMaxCount;
extern CFG_Tool *CFG_tools;
extern int CFG_tool; /// chosen tool

extern CFG_Param *CFG_params;
extern char *CFG_configFile;
extern char *CFG_helpFile;


#define CFG_renew(ptr, type, count, size) \
	if ((count) >= (size)) { size = max(10, (size)*2); ptr = renew_(ptr, type, size); }

#define CFG_allocData(param) \
	switch (param.type) { \
		case pInt: case pEnum: param.data = (void*) new_(int, param.size); break; \
		case pBool: param.data = (void*) new_(bool, param.size); break; \
		case pReal: param.data = (void*) new_(real, param.size); break; \
		case pString: case pFile: case pConfig: param.data = (void*) new_(char*, param.size); break; \
		default: break; \
	}
	
#define CFG_checkDataSize(name_, type_, size_) \
	switch (type_) { \
		case pInt: case pEnum: if (sizeof(typeof(name_)) != sizeof(int)*size_) ERROR_C("type mismatch with %s (size %d instead of %d for type %s)", #name_, (int)(sizeof(typeof(name_))), (int)(sizeof(int))*size_, #type_); break; \
		case pBool: if (sizeof(typeof(name_)) != sizeof(bool)*size_) ERROR_C("type mismatch with %s (size %d instead of %d for type %s)", #name_, (int)(sizeof(typeof(name_))), (int)(sizeof(bool))*size_, #type_); break; \
		case pReal: if (sizeof(typeof(name_)) != sizeof(real)*size_) ERROR_C("type mismatch with %s (size %d instead of %d for type %s)", #name_, (int)(sizeof(typeof(name_))), (int)(sizeof(real))*size_, #type_); break; \
		case pString: if (sizeof(typeof(name_)) != sizeof(char*)*size_) ERROR_C("type mismatch with %s (size %d instead of %d for type %s)", #name_, (int)(sizeof(typeof(name_))), (int)(sizeof(char*))*size_, #type_); break; \
		default: break; \
	}
	
	
/**
add a separator
*/
#define CFG_addOptionSep() \
	CFG_renew(CFG_tools[CFG_toolCount-1].options, CFG_Param, CFG_tools[CFG_toolCount-1].optionCount, CFG_tools[CFG_toolCount-1].optionMaxCount) \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].type = pSep; \
	CFG_tools[CFG_toolCount-1].optionCount++;

/**
add a tool
*/
#define CFG_addTool(name_, configType_, defaultConfigFile_, desc_) \
	CFG_renew(CFG_tools, CFG_Tool, CFG_toolCount, CFG_toolMaxCount) \
	CFG_tools[CFG_toolCount].name = name_; \
	CFG_tools[CFG_toolCount].desc = desc_; \
	CFG_tools[CFG_toolCount].configType = configType_; \
	CFG_tools[CFG_toolCount].options = NULL; \
	CFG_tools[CFG_toolCount].optionCount = 0; \
	CFG_tools[CFG_toolCount].optionMaxCount = 0; \
	CFG_tools[CFG_toolCount].optionsOwner = true; \
	for(int i=0; i<CFG_toolCount; i++) \
		if (strcmp(CFG_tools[i].configType, CFG_tools[CFG_toolCount].configType)==0) \
			{ CFG_tools[CFG_toolCount].options = CFG_tools[i].options; \
				CFG_tools[CFG_toolCount].optionCount = CFG_tools[i].optionCount; \
				CFG_tools[CFG_toolCount].optionsOwner = false; \
				break; } \
	CFG_toolCount++;

/**
get a tool from its name
*/	
extern CFG_Tool* CFG_getTool(const char *name);
/**
checks if the current tool (chosen by the user) is this one
*/
extern bool CFG_toolIs(const char *name);

/** 
add a parameter
*/

#define CFG_addOptionTMP(nameOpt_, nameVar_, type_, size_, default_, desc_, range_) \
	CFG_checkDataSize(nameVar_, type_, size_); \
	if (!CFG_tools[CFG_toolCount-1].optionsOwner) ERROR_C("Cannot add option to %s tool because not the owner of the options (have the same options that another tool", CFG_tools[CFG_toolCount-1].name); \
	CFG_renew(CFG_tools[CFG_toolCount-1].options, CFG_Param, CFG_tools[CFG_toolCount-1].optionCount, CFG_tools[CFG_toolCount-1].optionMaxCount); \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].data = &nameVar_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].type = type_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].size = size_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].name = #nameOpt_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].desc = desc_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].val = 0; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].default_val = default_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].range = range_; \
	CFG_tools[CFG_toolCount-1].options[CFG_tools[CFG_toolCount-1].optionCount].str = NULL; \
	CFG_setVal(CFG_tools[CFG_toolCount-1].options+CFG_tools[CFG_toolCount-1].optionCount, default_); \
	CFG_tools[CFG_toolCount-1].optionCount++;

/** 
add an option stored as a variable
*/
#define CFG_addOption(name_, type_, size_, default_, desc_, range_) \
	CFG_addOptionTMP(name_, name_, type_, size_, default_, desc_, range_)
/** 
add an option stored as a structure member
*/
#define CFG_addOptionS(struct_, name_, type_, size_, default_, desc_, range_) \
	CFG_addOptionTMP(name_, ((struct_).name_), type_, size_, default_, desc_, range_)

	

bool CFG_loadArgs(int argc, char **argv);
bool CFG_initOtherTool(const char *tool, char *config);
void CFG_printOptions();
#ifdef _USE_XML_
void CFG_writeOptionsXml(char *type, FILE *file);
#endif

void CFG_setVal(CFG_Param *param, char *arg);



#ifdef __cplusplus
	}
#endif


#endif



