/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#include "cyril-timer.h"

//////////////////////////////////////////////////////////////////
// timer
//////////////////////////////////////////////////////////////////

TimeStamp ts;

// get calendar seconds
Date get_c_s() 
{
	gettimeofday(&ts.tv,&ts.tz);
	return (ts.tv.tv_sec - ts.startingValue);
}
// get calendar milliseconds
Date get_c_ms() 
{
	gettimeofday(&ts.tv,&ts.tz);
	return (ts.tv.tv_sec - ts.startingValue)*1000 + ts.tv.tv_usec/1000;
}
// get calendar microseconds
Date get_c_us() 
{
	gettimeofday(&ts.tv,&ts.tz);
	return (ts.tv.tv_sec - ts.startingValue)*1000000 + ts.tv.tv_usec;
}

// get user cpu clocks
Date get_u_clk()
{
	times(&ts.ts);
	return ts.ts.tms_utime;
}
// get user cpu microseconds
Date get_u_us()
{
	return (Date)(get_u_clk()*1.E6 / CLOCKS_PER_SEC);
}

// get system cpu clocks
Date get_s_clk() 
{
	times(&ts.ts);
	return ts.ts.tms_stime;
}
// get system cpu microseconds
Date get_s_us()
{
	return (Date)(get_s_clk()*1.E6 / CLOCKS_PER_SEC);
}


// get user and system cpu clocks
Date get_us_clk()
{
//	times(&ts.ts);
//	return ts.ts.tms_stime + ts.ts.tms_utime;
	return times(&ts.ts);
}
// get user and system cpu microseconds
Date get_us_us() //;
{
	return (Date)(get_us_clk()*1.E6 / CLOCKS_PER_SEC);
}



void timer_start(Timer *c)
{
	times(&ts.ts);
	gettimeofday(&ts.tv,&ts.tz);
	c->calendar = (ts.tv.tv_sec - ts.startingValue)*1000000 + ts.tv.tv_usec;
//	c->user_cpu = (Date)(ts.ts.tms_utime*1.E6 / CLOCKS_PER_SEC);
//	c->system_cpu = (Date)(ts.ts.tms_stime*1.E6 / CLOCKS_PER_SEC);
	c->user_cpu = (Date)ts.ts.tms_utime;
	c->system_cpu = (Date)ts.ts.tms_stime;
}

void timer_stop(Timer *c)
{
	gettimeofday(&ts.tv,&ts.tz);
	times(&ts.ts);
	c->calendar = ((ts.tv.tv_sec - ts.startingValue)*1000000 + ts.tv.tv_usec) - c->calendar;
//	c->user_cpu = (Date)(ts.ts.tms_utime*1.E6 / CLOCKS_PER_SEC) - c->user_cpu;
//	c->system_cpu = (Date)(ts.ts.tms_stime*1.E6 / CLOCKS_PER_SEC) - c->system_cpu;
	c->user_cpu = (Date)ts.ts.tms_utime - c->user_cpu;
	c->system_cpu = (Date)ts.ts.tms_stime - c->system_cpu;
}

void timer_add(Timer *moy, Timer *c)
{
	moy->calendar += c->calendar;
	moy->user_cpu += c->user_cpu;
	moy->system_cpu += c->system_cpu;
}
/*
void timer_best(Timer *best, Timer *c)
{
	if (best->vals == 1)
	{
		if (c->calendar < best->calendar) best->calendar = c->calendar;
		if (c->user_cpu < best->user_cpu) best->user_cpu = c->user_cpu;
		if (c->system_cpu < best->system_cpu) best->system_cpu = c->system_cpu;
	} else
	{
		best->calendar = c->calendar;
		best->user_cpu = c->user_cpu;
		best->system_cpu = c->system_cpu;
		best->vals = 1;
	}
}
*/
void chrono_start(Chrono *c)
{
	timer_start(&c->timer);
}
void chrono_stop(Chrono *c)
{
	timer_stop(&c->timer);
	timer_add(&c->timer_tot, &c->timer);
}
void chrono_reset(Chrono *c)
{
	c->timer_tot = (Timer){0,0,0};
	c->vals = 0;
}

void chrono_norm(Chrono *c)
{
	c->timer_tot.calendar /= c->vals;
	c->timer_tot.user_cpu /= c->vals;
	c->timer_tot.system_cpu /= c->vals;
	c->vals = 1;
}

void chrono_print(Chrono *c)
{
	fprintf(stdout, "wc %lld, user %lld, sys %lld\n", 
		c->timer_tot.calendar, c->timer_tot.user_cpu, c->timer_tot.system_cpu);
}

/*

void chrono_start(Chrono *c)
{
//	c->user_cpu = times(&ts.ts);
	times(&ts.ts);
	gettimeofday(&ts.tv,&ts.tz);
	c->calendar = (ts.tv.tv_sec - ts.startingValue)*1000000 + ts.tv.tv_usec;
//	c->user_cpu = (Date)(ts.ts.tms_utime*1.E6 / CLOCKS_PER_SEC);
//	c->system_cpu = (Date)(ts.ts.tms_stime*1.E6 / CLOCKS_PER_SEC);
	c->user_cpu = (Date)ts.ts.tms_utime;
	c->system_cpu = (Date)ts.ts.tms_stime;
	c->vals = 0;
}

void chrono_stop(Chrono *c)
{
	gettimeofday(&ts.tv,&ts.tz);
	times(&ts.ts);
//	c->user_cpu = times(&ts.ts) - c->user_cpu;
	c->calendar = ((ts.tv.tv_sec - ts.startingValue)*1000000 + ts.tv.tv_usec) - c->calendar;
//	c->user_cpu = (Date)(ts.ts.tms_utime*1.E6 / CLOCKS_PER_SEC) - c->user_cpu;
//	c->system_cpu = (Date)(ts.ts.tms_stime*1.E6 / CLOCKS_PER_SEC) - c->system_cpu;
	c->user_cpu = (Date)ts.ts.tms_utime;
	c->system_cpu = (Date)ts.ts.tms_stime;
	c->vals = 1;
}

void chrono_print(Chrono *c)
{
	fprintf(stdout, "wc %lld, user %lld, sys %lld\n", c->calendar, c->user_cpu, c->system_cpu);
}

void chrono_init(Chrono *c)
{
	c->vals = 0;
	c->calendar = 0;
	c->user_cpu = 0;
	c->system_cpu = 0;
}


void chrono_best(Chrono *best, Chrono *c)
{
	if (best->vals == 1)
	{
		if (c->calendar < best->calendar) best->calendar = c->calendar;
		if (c->user_cpu < best->user_cpu) best->user_cpu = c->user_cpu;
		if (c->system_cpu < best->system_cpu) best->system_cpu = c->system_cpu;
	} else
	{
		best->calendar = c->calendar;
		best->user_cpu = c->user_cpu;
		best->system_cpu = c->system_cpu;
		best->vals = 1;
	}
}

void chrono_sum(Chrono *moy, Chrono *c)
{
	moy->calendar += c->calendar;
	moy->user_cpu += c->user_cpu;
	moy->system_cpu += c->system_cpu;
	moy->vals++;
}

void chrono_norm(Chrono *c)
{
	c->calendar /= c->vals;
	c->user_cpu /= c->vals;
	c->system_cpu /= c->vals;
	c->vals = 1;
}
*/
