/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#include "cyril-xml.h"

//////////////////////////////////////////////////////////////////
// xml high level
//////////////////////////////////////////////////////////////////
// void __() {}

//#include <libxml/tree.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

int xmlErr = 0;
bool xmlPrintErrors = true;

bool xmlPrintErrorsPile[20];
int xmlPrintErrorsCur = 0; // where to add in the pile

void xmlSetPrintErrors(bool val)
{
	if (xmlPrintErrorsCur < 20) 
		xmlPrintErrorsPile[xmlPrintErrorsCur++] = xmlPrintErrors;
	xmlPrintErrors = val;
}

void xmlRestorePrintErrors()
{
	if (xmlPrintErrorsCur > 0)
		xmlPrintErrors = xmlPrintErrorsPile[xmlPrintErrorsCur--];
}

//### keys


bool xmlKeyEq(xmlNodePtr cur, const char *keyName)
{
	return (cur && !xmlStrcmp(cur->name, (const xmlChar *)keyName));
}

xmlNodePtr xmlGotoNext(xmlNodePtr cur)
{
	do { cur = cur->next; }
	while (cur != NULL && (xmlKeyEq(cur, "text") || xmlKeyEq(cur, "comment")));
	return cur;
}

xmlNodePtr xmlGotoChildren(xmlNodePtr cur)
{
	cur = cur->xmlChildrenNode;
	while (cur != NULL && (xmlKeyEq(cur, "text") || xmlKeyEq(cur, "comment")))
		{ cur = cur->next; }
	return cur;
}

xmlNodePtr xmlFindKey(xmlNodePtr cur, const char *keyName)
{
	while (cur != NULL && !xmlKeyEq(cur, keyName))
		{ cur = cur->next; }
	return cur;
}

xmlNodePtr xmlFindNextKey(xmlNodePtr cur, const char *keyName)
{
	do { cur = cur->next; }
	while (cur != NULL && !xmlKeyEq(cur, keyName));
	return cur;
}


//### props

bool xmlPropExists(xmlNodePtr cur, const char *propName)
{
	return (bool)xmlGetProp(cur, (const xmlChar*)propName);
}

bool xmlPropEq(xmlNodePtr cur, const char *propName, const char *propVal)
{
	xmlErr = 0;
	bool res;
	char *s = (char*) xmlGetProp(cur, (const xmlChar*)propName);
	if (s == NULL) { xmlErr = -1; if (xmlPrintErrors) ERROR("Property \"%s\" doesn't exist in key \"%s\"", propName, cur->name); return false; }
	res = !xmlStrcmp((const xmlChar *)s, (const xmlChar *)propVal);
	xmlFree(s);
	return res;
}

char* xmlGetStrProp(xmlNodePtr cur, const char *propName)
{
	char *res = (char*) xmlGetProp(cur, (const xmlChar*)propName);
	xmlErr = (res ? 0 : -1);
	return res;
}

int xmlGetIntProp(xmlNodePtr cur, const char *propName)
{
	int res;
	char *val = (char *) xmlGetProp(cur, (const xmlChar*)propName);
	if (val == NULL) { xmlErr = -1; if (xmlPrintErrors) ERROR("Property \"%s\" doesn't exist in key \"%s\"", propName, cur->name); return 0; }
	res = (int)strtol(val, (char**)NULL, 10);//(int)atoi(val);
	xmlErr = errno;
	xmlFree(val);
	return res;
}

real xmlGetRealProp(xmlNodePtr cur, const char *propName)
{
	real res;
	char *val = (char *) xmlGetProp(cur, (const xmlChar*)propName);
	if (val == NULL) { xmlErr = -1; if (xmlPrintErrors) ERROR("Property \"%s\" doesn't exist in key \"%s\"", propName, cur->name); return 0; }
	res = (real)strtod(val, (char**)NULL);//(real)atof(val);
	xmlErr = errno;
	xmlFree(val);
	return res;
}

//### content

char* xmlGetStrCont(xmlDocPtr doc, xmlNodePtr cur)
{
	return (char*) xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
}

int xmlGetIntCont(xmlDocPtr doc, xmlNodePtr cur)
{
	int res;
	char *val = (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	if (val == NULL) { xmlErr = -2; if (xmlPrintErrors) ERROR("No content in key \"%s\"", cur->name); return 0; }
	res = (int)strtol(val, (char**)NULL, 10);//(int)atoi(val);
	xmlErr = errno;
	xmlFree(val);
	return res;
}

real xmlGetRealCont(xmlDocPtr doc, xmlNodePtr cur)
{
	real res;
	char *val = (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	if (val == NULL) { xmlErr = -2; if (xmlPrintErrors) ERROR("No content in key \"%s\"", cur->name); return 0; }
	res = (real)strtod(val, (char**)NULL);//(real)atof(val);
	xmlErr = errno;
	xmlFree(val);
	return res;
}

//### file

bool xmlOpenFile(const char *xmlname, xmlDocPtr *doc, xmlNodePtr *cur, const char *type)
{
	xmlErr = 0;
	if (!fexists(xmlname)) return false;
	*doc = xmlParseFile(xmlname); 
	if (*doc == NULL) { xmlErr = 1; if (xmlPrintErrors) ERROR("Failed opening Xml file \"%s\"", xmlname); return false; }
	*cur = xmlDocGetRootElement(*doc);
	if (*cur == NULL) { xmlErr = 2; if (xmlPrintErrors) ERROR("Wrong document type Xml file \"%s\"", xmlname); xmlFreeDoc(*doc); return false; }
	if (!xmlKeyEq(*cur, type)) { if (xmlPrintErrors) ERROR("Wrong document type Xml file \"%s\"", xmlname); xmlFreeDoc(*doc); return false; }
	*cur = xmlGotoChildren(*cur);
	return true;
}


void xmlCloseFile(xmlDocPtr doc)
{
	xmlFreeDoc(doc);
}
